<?php

namespace Ulmus\Api;

use Psr\Http\Message\RequestInterface;
use Psr\Http\Message\ResponseInterface;
use Ulmus\Adapter\AdapterInterface;
use Ulmus\Api\Request\JsonRequest;
use Ulmus\Api\Response\JsonResponse;
use Ulmus\Api\Response\Response;
use Ulmus\Ulmus;

class ConnectionAdapter extends \Ulmus\ConnectionAdapter
{
    public string $name;

    public array $configuration;

    protected AdapterInterface $adapter;

    private array $connection;

    public readonly ApiHandlerInterface $apiHandler;

    public function __construct(ApiHandlerInterface $apiHandler, $name = "default", array $configuration = [], bool $default = false)
    {
        parent::__construct($name, $configuration, $default);

        $this->apiHandler = $apiHandler;
    }


    public function resolveConfiguration(): void
    {
        $connection = $this->configuration['connections'][$this->name] ?? [];

        if (false !== ($adapterName = $connection['adapter'] ?? false)) {
            $this->adapter = $this->instanciateAdapter($adapterName);
        } else {
            $this->adapter = new Adapter\Rest();
        }

        $this->adapter->setup($connection);
    }

    public function getConfiguration(): array
    {
        return $this->configuration['connections'][$this->name];
    }

    /**
     * Connect the adapter
     * @return self
     */
    public function connect(): self
    {
        $this->connection = $this->adapter->connect();

        return $this;
    }

    public function connector(): object
    {
        return isset($this->connection) ? $this->connection : $this->connect()->connection;
    }

    public function adapter(): AdapterInterface
    {
        return $this->adapter;
    }

    protected function instanciateAdapter($name): AdapterInterface
    {
        $class = substr($name, 0, 2) === "\\" ? $name : "\\Ulmus\\Api\\Adapter\\$name";

        return new $class();
    }

    #public function apiResponseCallback(callable $callback = null) : callable|null
    #{
    #    return $callback ? $this->apiResponseCallback = $callback : $this->apiResponseCallback ?? fn(ResponseInterface $response) => $response;
    #}

    #public function apiRequestCallback(callable $callback = null) : callable|null
    #{
    #    return $callback ? $this->apiRequestCallback = $callback : $this->apiRequestCallback ?? fn(RequestInterface $request) => $request;
    #}
}
